#include <QApplication>
#include <QComboBox>
#include <QDebug>
#include <QFormLayout>
#include <QLabel>
#include <QShortcut>
#include <QStyle>

#include <QDebug>
#include <QSysInfo>

#include <algorithm>
#include <iostream>

// 1.2

static QString styleTitle()
{
    QString t;
#if QT_VERSION > 0x050000
    t += QGuiApplication::platformName() + QLatin1Char(' ');
#endif
    t += QApplication::style()->objectName() + QLatin1String("-Style");
    return t;
}

static QString title()
{
    return QLatin1String(QT_VERSION_STR) + QLatin1Char(' ') + styleTitle();
}

QComboBox *createCombo(const QString &on, bool edt)
{
    static QStringList items;
    if (items.isEmpty()) {
        for (int i = 11; i < 77; i += 11 )
            items +=QString::number(i);
    }
    auto cbox = new QComboBox();
    cbox->setEditable(edt);
    cbox->setObjectName(on);
    cbox->addItems(items);
    return cbox;
}

QComboBox *createStyledCombo(const QString &on, bool edt, const QString &css)
{
    auto cbox = createCombo(on, edt);
    cbox->setStyleSheet(css);
    return cbox;
}

int main(int argc, char *argv[])
{
    QStringList args;
    std::copy(argv + 1, argv + argc, std::back_inserter(args));

#if QT_VERSION > 0x050000
    QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
#endif

    QApplication app(argc, argv);

    QWidget w;
    w.setObjectName("TopLevel");
    QString t = title();
    qDebug()
#if QT_VERSION > 0x050000
        .noquote()
#endif
        << t << args;
    w.setWindowTitle(t);

    auto fl = new QFormLayout(&w);

    fl->addRow(new QLabel(styleTitle()));

    QString css = "QComboBox::!editable{background: green;}QComboBox::editable {background: yellow;}";
    fl->addRow(new QLabel(css));
    fl->addRow("Non-Editable", createStyledCombo("Non-Editable", false, css));
    fl->addRow("Editable", createStyledCombo("Editable", true, css));

    // workaround
    QString workAroundCss = "QComboBox[editable = \"false\"]{background: green;}QComboBox[editable=\"true\"] {background: yellow;}";
    fl->addRow(new QLabel(workAroundCss));
    fl->addRow("Non-Editable", createStyledCombo("Non-Editable-workaround", false, workAroundCss));
    fl->addRow("Editable", createStyledCombo("Editable-workaround", true, workAroundCss));

    fl->addRow(new QLabel("Plain"));
    fl->addRow("Non-Editable", createCombo("Non-Editable-Plain", false));
    fl->addRow("Editable", createCombo("Non-Editable-Plain", true));

    w.show();
    return app.exec();
}
